@@ -1,5 +1,5 @@
 ## comp_util.R | riskyr
-## 2022 08 14
+## 2025 07 24
 ## Generic utility functions:
 ## -----------------------------------------------
 
@@ -8,6 +8,9 @@
 ## (C) Conversion functions
 ## (D) Color and plotting functions (mostly moved to plot_util.R)
 ## (E) Text functions
+## (F) Numeric and numeral functions
+## (X) Miscellaneous
+
 
 ## (A) Verification functions: ------
 
@@ -18,13 +21,14 @@
 #  +. is_suff_freq_set      (ToDo)
 #  5. is_complement         (exported)
 #  6. is_prob_range         (NOT exported)
-#  7. is_extreme_prob_set   (exported)
-#  8. is_valid_prob_pair    (exported)
-#  9. is_valid_prob_set     (exported)
-# 10. is_valid_prob_triple  [exported, but deprecated]
-# 11. is_matrix             (exported)
+#  7. is_integer            (exported)
+#  8. is_extreme_prob_set   (exported)
+#  9. is_valid_prob_pair    (exported)
+# 10. is_valid_prob_set     (exported)
+# 11. is_valid_prob_triple  [exported, but deprecated]
+# 12. is_matrix             (exported)
 
-## is_prob: Verify that input is a probability ------
+# is_prob: Verify that input is a probability ------
 
 #' Verify that input is a probability (numeric value from 0 to 1).
 #'
@@ -87,7 +91,7 @@ is_prob <- function(prob, NA_warn = FALSE) {
     val <- FALSE
 
     if (NA_warn) {
-      warning(paste0(prob, " contains NA values. "))
+      warning(paste0(paste(prob, collapse = ", "), " contains NA values. "))
     }
   }
 
@@ -96,7 +100,7 @@ is_prob <- function(prob, NA_warn = FALSE) {
   #   val <- FALSE
   #
   #   if (NA_warn) {
-  #     warning(paste0(prob, " contains NaN values. "))
+  #     warning(paste0(paste(prob, collapse = ", "), " contains NaN values. "))
   #   }
   # }
 
@@ -105,7 +109,7 @@ is_prob <- function(prob, NA_warn = FALSE) {
     val <- FALSE
 
     if (NA_warn) {
-      warning(paste0(prob, " contains non-numeric values. "))
+      warning(paste0(paste(prob, collapse = ", "), " contains non-numeric values. "))
     }
   }
 
@@ -114,7 +118,7 @@ is_prob <- function(prob, NA_warn = FALSE) {
     val <- FALSE
 
     if (NA_warn) {
-      warning(paste0(prob, " contains values beyond the range from 0 to 1. "))
+      warning(paste0(paste(prob, collapse = ", "), " contains values beyond the range from 0 to 1. "))
     }
   }
 
@@ -130,8 +134,8 @@ is_prob <- function(prob, NA_warn = FALSE) {
 ## Check:
 # ## ways to succeed:
 # is_prob(1/2)                  # => TRUE
-# p.seq <- seq(0, 1, by = .1)   # Define vector of probabilities.
-# is_prob(p.seq)                # => TRUE (for vector)
+# p_seq <- seq(0, 1, by = .1)   # Define vector of probabilities.
+# is_prob(p_seq)                # => TRUE (for vector)
 #
 # ## watch out for:
 # is_prob(NA)                   # => FALSE + NO warning!
@@ -145,7 +149,7 @@ is_prob <- function(prob, NA_warn = FALSE) {
 # is_prob("Laplace", NA_warn = TRUE) # => FALSE + warning (non-numeric values)
 
 
-## is_perc: Verify that input is a percentage ------
+# is_perc: Verify that input is a percentage ------
 
 #' Verify that input is a percentage (numeric value from 0 to 100).
 #'
@@ -196,38 +200,39 @@ is_perc <- function(perc) {
 
     val <- FALSE
 
-    warning(paste0(perc, " contains NA values. "))
+    warning(paste0(paste(perc, collapse = ", "), " contains NA values. "))
   }
 
   else if (sum(is.nan(perc)) > 0) {
 
     val <- FALSE
 
-    warning(paste0(perc, " contains NaN values. "))
+    warning(paste0(paste(perc, collapse = ", "), " contains NaN values. "))
   }
 
   else if (sum(!is.numeric(perc)) > 0) {
     val <- FALSE
-    warning(paste0(perc, " contains non-numeric values. "))
+    warning(paste0(paste(perc, collapse = ", "), " contains non-numeric values. "))
   }
 
-  else if (sum((perc < 0) || (perc > 100)) > 0) {
+  else if ((sum(perc < 0) > 0) || (sum(perc > 100) > 0)) {
 
     val <- FALSE
 
-    warning(paste0(perc, " contains values beyond the range from 0 to 100. "))
+    warning(paste0(paste(perc, collapse = ", "), " contains values beyond the range from 0 to 100. "))
   }
 
   else {  # one way to succeed:
 
     val <- TRUE
+
   }
 
   return(val)
 
 }
 
-## is_freq: Verify that input is a frequency -------
+# is_freq: Verify that input is a frequency -------
 
 #' Verify that input is a frequency (positive integer value).
 #'
@@ -275,28 +280,28 @@ is_freq <- function(freq) {
 
     val <- FALSE
 
-    warning(paste0(freq, " contains NA values. "))
+    warning(paste0(paste(freq, collapse = ", "), " contains NA values. "))
   }
 
   else if (sum(is.nan(freq)) > 0) {
 
     val <- FALSE
 
-    warning(paste0(freq, " contains NaN values. "))
+    warning(paste0(paste(freq, collapse = ", "), " contains NaN values. "))
   }
 
   else if (sum(!is.numeric(freq)) > 0) {
 
     val <- FALSE
 
-    warning(paste0(freq, " contains non-numeric values. "))
+    warning(paste0(paste(freq, collapse = ", "), " contains non-numeric values. "))
   }
 
   else if (sum((freq < 0)) > 0) {
 
     val <- FALSE
 
-    warning(paste0(freq, " contains negative values (< 0). "))
+    warning(paste0(paste(freq, collapse = ", "), " contains negative values (< 0). "))
   }
 
   # else if (!all.equal(freq, as.integer(freq))) {
@@ -304,7 +309,7 @@ is_freq <- function(freq) {
 
     val <- FALSE
 
-    warning(paste0(freq, " contains non-integer values. "))
+    warning(paste0(paste(freq, collapse = ", "), " contains non-integer values. "))
   }
 
   else { # one way to succeed:
@@ -318,7 +323,7 @@ is_freq <- function(freq) {
 } # is_freq().
 
 
-## is_suff_prob_set: Verify that sufficient set of probabilities is provided ------
+# is_suff_prob_set: Verify that sufficient set of probabilities is provided ------
 
 #' Verify a sufficient set of probability inputs.
 #'
@@ -467,7 +472,7 @@ is_suff_prob_set <- function(prev,
 
 
 
-## is_complement: Verify that 2 numbers are complements -------
+# is_complement: Verify that 2 numbers are complements -------
 
 #' Verify that two numbers are complements.
 #'
@@ -593,7 +598,7 @@ is_complement <- function(p1, p2, tol = .01) {
 # # is_complement(8, 8)            # => FALSE + warning (beyond tolerance)
 
 
-## is_prob_range: Verify that some_range includes exactly 2 numeric prob values (from 0 to 1): ------
+# is_prob_range: Verify that some_range includes exactly 2 numeric prob values (from 0 to 1): ------
 
 is_prob_range <- function(some_range) {
 
@@ -690,11 +695,12 @@ is_integer <- function(x, tol = .Machine$double.eps^0.5) {
 # is.integer(1+2)
 # is_integer(1+2)
 
+
 ## (B) Beware of extreme cases: ------
 ##     Verify if the current set of (sufficient) probabilities
 ##     describe an extreme case:
 
-## is_extreme_prob_set: Verify that a prob set is an extreme case ------
+# is_extreme_prob_set: Verify that a prob set is an extreme case ------
 
 #' Verify that a set of probabilities describes an extreme case.
 #'
@@ -946,7 +952,7 @@ is_extreme_prob_set <- function(prev,
 # plot_tree(prev, 1, NA, 0, NA, N = 650)     # => illustrates this case
 
 
-## is_valid_prob_pair: Verify a pair of probability inputs -------
+# is_valid_prob_pair: Verify a pair of probability inputs -------
 
 # Verify that 2 probabilities are valid inputs
 # for a pair of complementary probabilities:
@@ -1051,7 +1057,7 @@ is_valid_prob_pair <- function(p1, p2, tol = .01) {
 # is_valid_prob_pair(c(.301, .299), .7)   # => TRUE
 
 
-## is_valid_prob_set: Verify a set of probability inputs ------
+# is_valid_prob_set: Verify a set of probability inputs ------
 
 # Verify that a set of up to 5 probabilities can
 # be interpreted as valid probability inputs:
@@ -1205,7 +1211,7 @@ is_valid_prob_set <- function(prev,
 # is_valid_prob_set(1, 1, 0, 1, 1)      # => FALSE + warning (beyond complement range)
 
 
-## is_valid_prob_triple: Verify a triple of essential probability inputs -------
+# is_valid_prob_triple: Verify a triple of essential probability inputs -------
 
 # Verify that a triple of inputs can
 # be interpreted as valid set of 3 essential probabilites:
@@ -1290,7 +1296,7 @@ is_valid_prob_triple <- function(prev, sens, spec) {
 # # is_valid_prob_triple(0, NA, 0)   # => FALSE + warning (NA)
 # # is_valid_prob_triple("p", 0, 0)  # => FALSE + warning (non-numeric)
 
-## is_matrix: Verify that mx is a numeric 2x2 contingency table: ------
+# is_matrix: Verify that mx is a numeric 2x2 contingency table: ------
 
 #' Verify a 2x2 matrix as a numeric contingency table.
 #'
@@ -1364,6 +1370,7 @@ is_matrix <- function(mx){
 
   } # if end.
 
+
   return(out)
 
 } # is_matrix().
@@ -1379,11 +1386,12 @@ is_matrix <- function(mx){
 # is_matrix(as.table(matrix(0:3, nrow = 2, ncol = 2)))
 
 
+
 ## (C) Conversion functions: --------
 
 ## Toggle between showing probabilities and percentages:
 
-## as_pc: Show a probability as a (numeric and rounded) percentage ------
+# as_pc: Show a probability as a (numeric and rounded) percentage ------
 
 #' Display a probability as a (numeric and rounded) percentage.
 #'
@@ -1430,6 +1438,7 @@ as_pc <- function(prob, n_digits = 2) {
 
   perc <- NA # initialize
 
+
   if (is_prob(prob)) {
 
     perc <- round(prob * 100, n_digits)  # compute percentage
@@ -1444,6 +1453,7 @@ as_pc <- function(prob, n_digits = 2) {
 
   }
 
+
   return(perc)  # return (numeric)
 
 } # as_pc().
@@ -1457,6 +1467,7 @@ as_pc <- function(prob, n_digits = 2) {
 # as_pc(NA)
 # as_pc(0/0)
 
+
 ## Removed from documentation (to avoid ERRORS):
 #
 # ## ways to fail:
@@ -1474,9 +1485,10 @@ as_pc <- function(prob, n_digits = 2) {
 # # round(prob_seq, 4) == as_pb(as_pc(prob_seq))  # => all TRUE (both rounded to 4 decimals)
 
 
+
 ## Percentage as probability (4 decimals):
 
-## as_pb: Show a percentage as a (numeric and rounded) probability ------
+# as_pb: Show a percentage as a (numeric and rounded) probability ------
 
 #' Display a percentage as a (numeric and rounded) probability.
 #'
@@ -1517,8 +1529,10 @@ as_pc <- function(prob, n_digits = 2) {
 
 as_pb <- function(perc, n_digits = 4) {
 
+
   prob <- NA # initialize
 
+
   if (is_perc(perc)) {
 
     prob <- round(perc/100, n_digits) # compute
@@ -1530,9 +1544,10 @@ as_pb <- function(perc, n_digits = 4) {
 
   }
 
+
   return(prob)  # numeric value
 
-}
+} # as_pb().
 
 ## Check:
 # as_pb(1/3)          # => 0.0033
@@ -1562,17 +1577,8 @@ as_pb <- function(perc, n_digits = 4) {
 # # round(prob_seq, 4) == as_pb(as_pc(prob_seq))  # => all TRUE (both rounded to 4 decimals)
 
 
-## incsum: Incremental sum (as an inverse of cumsum): ------
 
-incsum <- function(cumsum){
 
-  diff(c(0, cumsum))
-
-}
-
-## Check:
-# v <- runif(10)
-# all.equal(incsum(cumsum(v)), v)
 
 
 
@@ -1583,7 +1589,7 @@ incsum <- function(cumsum){
 # - Use unikn pkg or functions for color settings.
 
 
-## make_transparent: Make colors transparent ----
+# make_transparent: Make colors transparent ----
 
 make_transparent <- function(..., alpha = .50) {
 
@@ -1604,7 +1610,7 @@ make_transparent <- function(..., alpha = .50) {
 
   return(new_color)
 
-}
+} # make_transparent().
 
 ## Check:
 # make_transparent(c("black", "red"), alpha =  0)
@@ -1615,13 +1621,20 @@ make_transparent <- function(..., alpha = .50) {
 # grDevices::adjustcolor(col = "green", alpha.f = .50)
 
 
+
+
 ## (E) Text functions: --------
 
+# capitalise_1st: Capitalize initial character of a string ------
+
 capitalise_1st <- function(string) {
+
   String <- ""
   String <- paste0(toupper(substr(string, 1, 1)), substr(string, 2, nchar(string)))
+
   return(String)
-}
+
+} # capitalise_1st().
 
 ## Check:
 # capitalise_1st("the end.") # "The end."
@@ -1629,7 +1642,530 @@ capitalise_1st <- function(string) {
 # capitalise_1st(123)        # "123"
 
 
-## (F) Miscellaneous: --------
+
+
+
+## (F) Numeric and numeral functions: --------
+
+
+
+# - A. Numeric and numeral sequences: --------
+
+# incsum: Incremental sum (as an inverse of cumsum): ------
+
+incsum <- function(cumsum){
+
+  diff(c(0, cumsum))
+
+} # incsum().
+
+## Check:
+# v <- runif(10)
+# all.equal(incsum(cumsum(v)), v)
+
+
+
+# tally: Count number of symbol occurrences in a character scalar (converted into a vector of elements of nchar = 1) ------
+
+tally_1 <- function(s, target = "1"){
+
+  # Verify inputs:
+  if ( !is.character(s) | length(s) > 1){
+    message("tally: s should be a scalar character object (of length 1)")
+  }
+
+  if ( !is.character(target) | length(target) > 1 | nchar(target) > 1 ){
+    message("tally: target should be a scalar character object (of 1 character)")
+  }
+
+  # Main:
+  s_v <- unlist(strsplit(s, split = ""))  # split into a vector of individual characters
+
+  sum(s_v == target)  # count
+
+} # tally_1().
+
+# Vectorized version:
+tally <- Vectorize(tally_1, vectorize.args = "s")
+
+# # Check:
+# tally("10101")
+# tally(c("010", "101", "111"))
+# tally(c("HTTH", "HTHTTH", "ABC"), target = "H")
+
+# # Tests:
+# chars <- paste(LETTERS, collapse = "")
+# unlist(strsplit(chars, split = ""))
+
+
+
+# - B. Base conversions: --------
+
+# base_dec: Use polynomial expansion to convert number represented in some base notation into decimal numerals: ------
+
+base_dec <- function(x, base = 2){
+
+  x <- as.numeric(x)  # x denotes value (in current notation)
+
+  # Prepare:
+  if (x < 0) (is_negative <- TRUE) else (is_negative = FALSE)   # remember
+  n <- as.character(abs(x))  # string of numerals
+  n_pos <- nchar(n)  # number of positions in base
+  d <- unlist(strsplit(n, split = "")) # vector of digits
+
+  tot_val <- 0  # initialize
+
+  # Main:
+  for (i in 1:n_pos){
+
+    # Current digit (as number):
+    cur_n <- as.numeric(d[n_pos - (i - 1)])
+
+    # Note illegal digits (given base):
+    if (cur_n > base - 1){
+      message(paste0("A digit of ", cur_n, " is invalid in base ", base))
+    }
+
+    # Expand polynomial:
+    cur_val <- cur_n  * (base ^ (i - 1))
+    tot_val <- tot_val + cur_val
+
+  }
+
+  # Output:
+  if (is_negative){ tot_val <- tot_val * -1}
+
+  return(tot_val)
+
+} # base_dec().
+
+# # Check:
+# base_dec(111, base = 2)
+# base_dec(111, base = 3)
+# base_dec(111, base = 10)
+# base_dec(-1010, base = 2)  # works for negative numbers
+# base_dec(123, base = 3)    # notes invalid numerals (but proceeds using them)
+
+
+
+# base_2_dec: Convert number represented in some base notation into decimal numerals -------
+#             using a recursive function.
+
+base_2_dec <- function(x, base = 2, exp = 0){
+
+  x <- as.numeric(x)  # x denotes value (in decimal notation)
+
+  if (x == 0) { # stop:
+
+    return(0)
+
+  } else { # simplify:
+
+    cur_dig <- (x %% 10^(exp + 1)) / 10^exp
+    cur_val <- cur_dig * base^exp
+
+    next_x <- x - cur_dig * 10^exp
+
+    return(cur_val + base_2_dec(x = next_x, base = base, exp = (exp + 1)))
+
+  }
+
+} # base_2_dec().
+
+# # Check:
+# base_2_dec(x = 11, base = 2)
+# base_2_dec(x = 11, base = 7)
+# base_2_dec(x = 101, base = 2)
+# base_2_dec(x = 222, base = 1)  # Note: Non-sensical inputs.
+
+
+# dec_2_base: Convert number represented in decimal numerals to notation in some other base -------
+
+# Recursive Version (with accuracy limit to 16-digit numeric representations):
+
+dec_2_base <- function(x, base = 2, exp = 0) {
+
+  # Warning:
+  fb <- TRUE  # user feedback
+  if (fb && (exp > 16)){
+    warning(paste0("Beware rounding inaccuracies for exp = ", exp))
+  }
+
+
+  if (x == 0) { # stop:
+
+    return(0)
+
+  } else { # simplify:
+
+    rest  <- x %%  base
+    nxt_x <- x %/% base
+
+    add_2 <- rest * (10^exp)
+
+    # recurse:
+    return(add_2 + dec_2_base(x = nxt_x, base = base, exp = (exp + 1)))
+
+  }
+
+} # dec_2_base().
+
+# # Check:
+# dec_2_base(x = 11, base = 2)
+# dec_2_base(x =  2, base = 2)
+# dec_2_base(x = 11, base = 4)
+# dec_2_base(x =  4, base = 3)
+# dec_2_base(x = 651361, base = 2)
+
+# Warning: Limit in accuracy / number of decimal positions:
+# Converting decimal values larger than 65535 to base 2
+# can yield errors, due to possible rounding inaccuracies.
+#
+# Note:
+# 65535 = 2^16 - 1
+# as.character(dec_2_base(x = 65535, base = 2)) # = "1111111111111111" (16 x "1")
+#
+# as.character(dec_2_base(65536 + 1, base = 2))
+# ds4psy::dec2base(65536 + 1, base = 2)
+#
+# Problem cases:
+# # Note some conflict/erroneous cases:
+# as.character(dec_2_base(68485, base = 2)) # but
+# ds4psy::dec2base(68485, base = 2)
+#
+# as.character(dec_2_base(73843, base = 2)) # (is impossible), but
+# ds4psy::dec2base(73843, base = 2)
+#
+# as.character(dec_2_base(76791, base = 2)) # (is impossible), but
+# ds4psy::dec2base(76791, base = 2)
+
+# More conflict cases:
+# 75437
+# 79761
+# 93019
+
+# Solution: The gmp package uses string representations:
+#           (see <https://CRAN.R-project.org/package=gmp>).
+
+
+# Alternative attempt: Provide result as character sequence (more precise than numeric for longer sequences):
+
+dec_2_base_alt <- function(x, base = 2, exp = 0){
+
+  # Prepare:
+  fb <- FALSE  # provide user feedback (4debugging)
+
+  x <- as.numeric(x)  # x denotes value (in decimal notation)
+
+  # Main:
+  if (x < base) { # stop:
+
+    if (fb){
+      message(paste0("Stop at x = ", x, ", ", "exp = ", exp, ": "))
+    }
+
+    if (x == 0){
+      return()  # no leading zero
+    } else {
+      return(as.character(x))  #
+    }
+
+  } else { # simplify:
+
+    cur_rem <- x %% (base^(exp + 1))
+    cur_dig <- cur_rem %/% (base^exp)
+    dig_val <- cur_dig  *  (base^exp)
+
+    # next_x <- x %/% base
+    next_x <- x - dig_val
+
+    # Feedback:
+    if (fb){
+      message(paste0("x = ", x, ", ", "exp = ", exp, ": "))
+      message(paste0("cur_rem = ", cur_rem, ", ",
+                     "cur_dig = ", cur_dig, ", ",
+                     "dig_val = ", dig_val, ", ",
+                     "next_x = ", next_x, "."))
+    }
+
+    # Recurse:
+    paste0(dec_2_base_alt(x = next_x, base = base, exp = (exp + 1)), cur_dig, collapse = "")
+
+  }
+
+} # dec_2_base_alt().
+
+# # Check:
+# dec_2_base_alt(x = 11, base = 2)
+
+# dec_2_base_alt(x =  2, base = 2)
+# dec_2_base_alt(x = 11, base = 4)
+# dec_2_base_alt(x =  4, base = 3)
+# dec_2_base_alt(x = 651361, base = 2)
+
+# # Correctly handles former problem cases:
+# as.character(dec_2_base(68485, base = 2)) # is wrong, but
+# dec_2_base_alt(68485, base = 2)           # equals
+# ds4psy::dec2base(68485, base = 2)
+#
+# as.character(dec_2_base(73843, base = 2))  # (is impossible), but
+# dec_2_base_alt(73843, base = 2)            # equals
+# ds4psy::dec2base(73843, base = 2)
+#
+# as.character(dec_2_base(76791, base = 2))      # (is impossible), but
+# as.character(dec_2_base_alt(76791, base = 2))  # equals
+# ds4psy::dec2base(76791, base = 2)
+#
+# # Note that sequences (interpreted as integers) cross R's limit of numeric accuracy:
+# dec_2_base_alt(68485, base = 2)
+# as.character(as.numeric(dec_2_base_alt(68485, base = 2)))
+#
+# dec_2_base_alt(73843, base = 2)
+# as.character(as.numeric(dec_2_base_alt(73843, base = 2)))
+#
+# dec_2_base_alt(76791, base = 2)
+# as.character(as.numeric(dec_2_base_alt(76791, base = 2)))
+#
+# nchar(dec_2_base_alt(68485, base = 2))
+# nchar(dec_2_base_alt(73843, base = 2))
+# nchar(dec_2_base_alt(76791, base = 2))
+
+
+# Questions:
+# - Which function is wrong? (Suspicion: `dec_2_nondec()`, or both)
+# - What's wrong with it? (Hypothesis: Numeric accuracy of R limited to 16 integers?)
+# - Why do conflicts only occur for base 2? (Suspicion: Low bases get longer more quickly.)
+
+# Lessons learned:
+# - Beware of numeric accuracy/limitations of R
+# - Use character sequences when dealing with strings of non-decimal digits
+
+
+
+
+
+
+# - C. Sequential percentage changes: --------
+
+# Task: Compute true (geometric) net (aggregate) change of a pcs.
+
+# pc_2_fac: Convert a series of percentage changes (pcs) into corresponding multiplicative factor(s) ------
+
+pc_2_fac <- function(pcs) {
+
+  # Verify percentage values:
+  is_perc(pcs)
+
+  # facs <- 1 + pcs/100
+  facs <- (100 + pcs)/100
+
+  return(facs)
+
+} # pc_2_fac().
+
+# Checks:
+# pc_2_fac(10)  # [1] 1.1
+# pc_2_fac(c(0, +5, +10, +25, +50, +75, +100, +200, +300, +500, +1000))
+# 1.00  1.05  1.10  1.25  1.50  1.75  2.00  3.00  4.00  6.00 11.00
+# pc_2_fac(c(0, -5, -10, -25, -50, -75, -100, -200, -300, -500, -1000))
+# 1.00  0.95  0.90  0.75  0.50  0.25  0.00 -1.00 -2.00 -4.00 -9.00
+
+
+
+# fac_2_pc: Reverse: Convert multiplicative factor(s) into corresponding percentage changes ------
+
+fac_2_pc <- function(facs) {
+
+  pcs <- (facs * 100) - 100
+
+  # Verify percentage values:
+  is_perc(pcs)
+
+  return(pcs)
+
+} # fac_2_pc().
+
+# # Checks:
+# fac_2_pc(c(1.00,  1.05,  1.10,  1.25,  1.50,  1.75,  2.00,  3.00,  4.00,  6.00, 11.00))
+# # 0    5   10   25   50   75  100  200  300  500 1000
+# fac_2_pc(c(1.00,  0.95,  0.90,  0.75,  0.50,  0.25,  0.00, -1.00, -2.00, -4.00, -9.00))
+# # 0    -5   -10   -25   -50   -75  -100  -200  -300  -500 -1000
+
+
+# +++ here now +++
+
+# aggr_pcs: Aggregate change of a series of percentage changes ------
+
+# Compute true (geometric) net (aggregate) change of a pcs with SAME overall effect (to replace):
+# True (geometric) net change of a pcs:
+
+aggr_pcs <- function(pcs) {
+
+  # Goal: Replace a pcs (a series of arbitrary percentage changes)
+  #       by 1 percentage change of y% with the SAME overall effect.
+
+  # initialize:
+  f_pcs <- NA
+  f_y <- NA
+  p_y <- NA
+  p_y_formula <- NA
+
+  # 1) Use 3 functions/steps:
+  f_pcs <- pc_2_fac(pcs)  # 1. convert pcs into corresponding multiplicative factors.
+  f_y   <- prod(f_pcs)    # 2. compute the product of all factors as a new factor f_y.
+  p_y   <- fac_2_pc(f_y)  # 3. convert resulting factor f_y into a percentage change.
+
+  ## Same 3 steps in one:
+  # p_y_2 <- fac_2_pc(prod(pc_2_fac(pcs)))
+
+  # 2) using formula (derived in "relativeChangePerception_yymmdd.Rmd"):
+  p_y_formula <- (100 * (prod((100 + pcs)/100))) - 100
+
+  # Test both solutions for equality:
+  if (!all.equal(p_y, p_y_formula)) {
+    warning("Result of 3 functions differs from formula result.")
+  }
+
+  return(p_y)
+
+} # aggr_pcs().
+
+# # Check:
+# aggr_pcs(33)                 #  33 (works for scalars)
+# aggr_pcs(c(10,  20))         #  32
+# aggr_pcs(c(50, -50))         # -25
+# aggr_pcs(c(50, -50, 100/3))  #   0 !
+# aggr_pcs(rep(10, 10))        # 159.3742
+#
+# # Example: Weekly changes over 1 year (52 weeks):
+# wc <- 10  # percentage of weekly change
+#
+# # Intuition: Does order matter? (Commutativity):
+# up_dn_52weeks <- rep(c(+wc, -wc), times = 26)  # start with 1 gain:
+# aggr_pcs(up_dn_52weeks)    # -22.99...
+#
+# up_first_half <- rep(c(+wc, -wc), each = 26)  # start with 26 gains:
+# aggr_pcs(up_first_half)    # -22.99...
+#
+# dn_up_52weeks <- rep(c(-wc, +wc), times = 26)  # start with 1 loss:
+# aggr_pcs(dn_up_52weeks)    # -22.99... (same)
+#
+# dn_first_half <- rep(c(-wc, +wc), each = 26)  # start with 26 losses:
+# aggr_pcs(dn_first_half)    # -22.99...
+
+
+
+## (X) Miscellaneous: --------
+
+
+
+## swap_xy_dims: Swap 2 dimensions x and y of an array/table OR elements of a vector/list/columns of a data.frame: ------
+
+# Imported code snippet (from the i2ds package):
+# Goal: A generalized version of t() that works for multiple object types.
+
+swap_xy_dims <- function(obj, x = 1, y = 2){
+
+  # Inputs: Ensure that x and y are integer values:
+  if ( (x %% 1 != 0) || (y %% 1 != 0) ) {
+
+    message("swap_xy_dims: x and y must be integer values.")
+    x <- as.integer(x)
+    y <- as.integer(y)
+
+  }
+
+  t_obj <- NA  # initialize
+
+
+  if (is.array(obj)) { # Case 1: obj is an array/matrix/table: ----
+
+    # print("1: An array, matrix, or table: Transpose dimensions")  # 4debugging
+
+    n_dim <- length(dim(obj))
+
+    if ( (x <= n_dim) && (y <= n_dim) && (x != y) ){
+
+      ix    <- 1:n_dim
+      ix[x] <- y
+      ix[y] <- x
+
+      t_obj <- aperm(obj, perm = ix)  # permute dimensions x and y
+
+    } else { # no change:
+
+      message("swap_xy_dims: no change.")
+      t_obj <- obj
+
+    }
+
+
+  } else if ( is.atomic(obj) | is.list(obj) ) { # Case 2: obj is atomic vector/list/df: ----
+
+    # print("2: An atomic vector, list, or data frame: Swap elements")  # 4debugging
+
+    n <- length(obj)
+
+    if ( (x <= n) && (y <= n) && (x != y) ){
+
+      ix    <- 1:n
+      ix[x] <- y
+      ix[y] <- x
+
+      t_obj <- obj[ix]  # switch elements x and y
+
+
+    } else { # Trivial case 0: no change:
+
+      message("swap_xy_dims: no change.")
+      t_obj <- obj
+
+    }
+
+
+  } else { # any other obj:
+
+    message("swap_xy_dims: obj is not an array/table or linear vector/list.")
+
+  }
+
+
+  return(t_obj)
+
+} # swap_xy_dims().
+
+# # Check:
+# (v  <- 1:4)
+# (m <- matrix(1:6, nrow = 3))
+# (l  <- list(a = 1:3, b = letters[1:2], c = 11:14))
+# (df <- data.frame(v1 = v, v2 = letters[1:4], v3 = 11:14))
+# (ar <- array(1:24, dim = c(4, 3, 2)))
+# (tb <- UCBAdmissions)
+#
+# swap_xy_dims(v)
+# swap_xy_dims(v, 1, 4)
+# swap_xy_dims(v, 2.2, pi)  # as integers
+# swap_xy_dims(v, 1, 9)     # no change
+#
+# swap_xy_dims(l)
+#
+# swap_xy_dims(l, 2, pi)
+# swap_xy_dims(l, 1, 9)     # no change
+#
+# swap_xy_dims(df) # df is list
+#
+# swap_xy_dims(m)  # same as t(m)
+#
+# swap_xy_dims(ar)
+# swap_xy_dims(ar, 2, 3)
+#
+# swap_xy_dims(tb)
+# swap_xy_dims(tb, 2, 3)
+# swap_xy_dims(tb, 1, pi)
+#
+# swap_xy_dims(factor(v)) # factor
+
+
 
 # kill_all: Kill all objects in current environment (without warning): ----
 
@@ -1637,11 +2173,12 @@ kill_all <- function(){
 
   rm(list = ls())
 
-}
+} # kill_all().
 
 # Check: ----
 # kill_all()
 
+
 ## (*) Done: --------
 
 ## - Clean up code [2021 03 20].
